"""Frontegg client and proxy."""
from frontegg.helpers import exceptions as frontegg_exceptions
from frontegg.helpers.logger import logger as frontegg_logger


__all__ = ('frontegg_exceptions', 'frontegg_logger')
