from _typeshed import Incomplete
from frontegg.common import FronteggAuthenticator as FronteggAuthenticator, IdentityClientMixin as IdentityClientMixin
from frontegg.common.frontegg_context import FronteggContext as FronteggContext
from typing import Optional

class Frontegg:
    authenticator: Incomplete
    identity_client: Incomplete
    def __init__(self) -> None: ...
    def init_app(self, client_id: str, api_key: str, options=...): ...
    @property
    def should_refresh_vendor_token(self) -> bool: ...
    @property
    def access_token(self): ...
    def refresh_vendor_token(self) -> None: ...
    @property
    def client_id(self): ...
    @property
    def api_key(self): ...
    def get_public_key(self) -> str: ...
    def fetch_public_key(self) -> str: ...
    def validate_identity_on_token(self, token, options, type): ...
    def decode_jwt(self, authorization_header, verify: Optional[bool] = ...): ...

frontegg: Incomplete
